// CPUView.cpp
// Copyright(C) 1999 by Kiyoshi Kobayashi. All rights reserved.
//
//
#include <Point.h>
#include <OS.h>
#include <Debug.h>
#include <stdio.h>
#include <string.h>

#include "CPUView.h"
#include "SysInfoDeltaGenerator.h"

static const char CLASS_NAME[] = "CPUView";

static const char VIEW_NAME[] = "CPUView";
static const char ID_NAME[] = "CPUView::id";

/* defines minimum&maximum size of CPU view */
static const BRect MIN_SIZE(0.0f, 0.0f, 60.0f, 20.0f);
static const BRect MAX_SIZE(0.0f, 0.0f, 1600.0f, 1200.0f);
static const float MARGIN = 1.0f;
static const float STR_VIEW_HEIGHT = 9.0f;
static const float FONT_SIZE = 9.0f;

// private observer class
//
class CPUSysInfoObserver : public SysInfoObserver {
  public:
	CPUSysInfoObserver(CPUView &peer) : view(peer), id(peer.GetId()) {
	};
	
	~CPUSysInfoObserver() {};
	
	inline void Updated(const SysInfo &delta) {
		float ar = ((float)delta.GetInfo().cpu_infos[id].active_time
					/ (float)delta.GetSystemTime());
		view.Update(ar);
	};
	
  private:
	CPUView &view;
	int id;
};

// Public: Constructor.
//
CPUView::CPUView(BRect frame, int32 cpuId)
	: BaseView(frame, VIEW_NAME, B_FOLLOW_ALL, B_WILL_DRAW|B_FRAME_EVENTS,
		   MIN_SIZE, MAX_SIZE), id(cpuId) {
	// Set the name of itself
	name[0] = '\0';
	sprintf(name, "cpu %ld", id);
	SetName(name);
	
	char buf[1024];
	buf[0] = '\0';
	
	// graph
	buf[0] = '\0';
	sprintf(buf, "%s line graph", name);
	gView = new LineGraphView(frame, buf, 1.0f);

	ASSERT(gView != NULL);
	AddChild(gView);

	// string
	buf[0] = '\0';
	sprintf(buf, "cpu(%s) string view", name);
	char str[64];
	str[0] = '\0';
	sprintf(str, "%s :", name);
	sView = new LabelStringView(frame, str, "0", "%");
	
	ASSERT(sView != NULL);
	AddChild(sView);
	
	// SysInfo observer
	obs = new CPUSysInfoObserver(*this);
	ASSERT(obs != NULL);
}

// Public: Constructor from the archive.
//
CPUView::CPUView(BMessage *archive) : BaseView(archive) {
	status_t ret;
	// Restore CPU id
	//
	ret = archive->FindInt32(ID_NAME, &id);
	ASSERT(ret == B_OK);
	
	// restore transient pointer to the children
	//
	BView *child = this->ChildAt(0);

	while (child != NULL) {
		if (is_instance_of(child, LineGraphView))
			gView = cast_as(child, LineGraphView);
		else if (is_instance_of(child, LabelStringView))
			sView = cast_as(child, LabelStringView);
		
		child = child->NextSibling();
	}
	
	ASSERT(gView != NULL);
	ASSERT(sView != NULL);

	// SysInfo observer
	obs = new CPUSysInfoObserver(*this);
	ASSERT(obs != NULL);
}

// Public: Destructor.
//
CPUView::~CPUView() {
	// Unregister SysInfo observer
	SysInfoDeltaGenerator *gen = SysInfoDeltaGenerator::GetGenerator();
	ASSERT(gen != NULL);
	
	gen->Unregister(obs);
	delete(obs);
	obs = NULL;
}

// Public: Override BView method.
//
void
CPUView::AttachedToWindow() {
	if (Parent()) {
		const rgb_color view = Parent()->ViewColor();
		SetViewColor(view);
	}
	
	// Adjust the size
	Adjust(Frame().Width(), Frame().Height());
	
	// Register SysInfo observer
	SysInfoDeltaGenerator *gen = SysInfoDeltaGenerator::GetGenerator();
	ASSERT(gen != NULL);
	
	gen->Register(obs);
	
	BaseView::AttachedToWindow();
}

// Public: Override BView method.
//
void
CPUView::FrameResized(float width, float height) {
	Adjust(width, height);

	BaseView::FrameResized(width, height);
}

// Public: Update the contents.
//
void
CPUView::Update(float rate) {
	char buf[1024];
	buf[0] = '\0';
	sprintf(buf, "%d",(int)(rate * 100.0f));
	gView->Update(rate);
	sView->Update(buf);
}

// Public: Archive the object.
//
status_t
CPUView::Archive(BMessage *archive, bool deep) const {
	status_t ret;
	ret = BaseView::Archive(archive, deep);
	ASSERT(ret == B_OK);
	
	ret = archive->AddString("class", CLASS_NAME);
	ASSERT(ret == B_OK);
	
	if ( deep ) { 
		ret = archive->AddInt32(ID_NAME, id);
		ASSERT(ret == B_OK);
	}

	return B_OK;
}

// Public/Static: Instanciate the object.
//
BArchivable *
CPUView::Instantiate(BMessage *archive) 
{
	if (validate_instantiation(archive, CLASS_NAME))
		return new CPUView(archive);
	
	return NULL; 
}

// Private:
//
CPUView::CPUView(const CPUView &)
	: BaseView(BRect(), VIEW_NAME, B_FOLLOW_ALL, B_WILL_DRAW, MIN_SIZE, MAX_SIZE) {}

// Private: Adjust the size.
//
void
CPUView::Adjust(float width, float height) {
	gView->MoveTo(MARGIN, MARGIN);
	gView->ResizeTo(width - MARGIN * 2.0f,
					height - STR_VIEW_HEIGHT - MARGIN);
	PRINT(("CPUView::Adjust(%s) ", name));
	PRINT_OBJECT(gView->Frame());

	sView->MoveTo(MARGIN, height + MARGIN - STR_VIEW_HEIGHT);
	sView->ResizeTo(width - MARGIN * 2.0f, STR_VIEW_HEIGHT);
	PRINT(("Resize string "));
	PRINT_OBJECT(sView->Frame());
}

